@@ -12,12 +12,16 @@ def define(text: str):
 
 
 if __name__ == "__main__":
-    from qtpy.QtWidgets import QApplication, QWidget
+    from qtpy import QtWidgets as QtW
 
-    app = QApplication([])
-    main = QWidget()
+    app = QtW.QApplication([])
+    main = QtW.QWidget()
     main.setMinimumSize(400, 300)
 
+    main.setLayout(QtW.QVBoxLayout())
+    _checkbox = QtW.QCheckBox("Check")
+    main.layout().addWidget(_checkbox)
+
     palette = get_palette(name=__name__)
     storage = get_storage(name=__name__)
 
@@ -31,11 +35,15 @@ if __name__ == "__main__":
     for txt in ["foo", "bar", "baz"]:
         group_0.register(define(txt))
 
+    group_0.register(define("only if checked"), when=_checkbox.isChecked)
+
     for txt in ["hello world", "goobye world", "hello again"]:
         group_1.register(define(txt))
 
+    group_1.register(define("only if checked"), when=_checkbox.isChecked)
+
     @group_1.register("Print widget")
-    def print_widget(widget: QWidget):
+    def print_widget(widget: QtW.QWidget):
         print(widget)
 
     palette.install(main, "Ctrl+Shift+P")
