@@ -10,22 +10,32 @@ if TYPE_CHECKING:
     from ._widget import QCommandPalette
     from qtpy import QtWidgets as QtW
 
+    WVDict = weakref.WeakValueDictionary[int, Any]
+
 _F = TypeVar("_F", bound=Callable)
 
 
+def _always_true() -> bool:
+    return True
+
+
 @inspect.signature
 def register_with_func(
     func: Callable,
     title: str | None = None,
     desc: str | None = None,
     tooltip: str | None = None,
+    when: Callable[[], bool] = _always_true,
 ):
     """Template function to provide signature to register() with 'func' argument."""
 
 
 @inspect.signature
 def register_without_func(
-    title: str | None = None, desc: str | None = None, tooltip: str | None = None
+    title: str | None = None,
+    desc: str | None = None,
+    tooltip: str | None = None,
+    when: Callable[[], bool] = _always_true,
 ):
     """Template function to provide signature to register() without 'func' argument."""
 
@@ -35,11 +45,13 @@ class CommandPalette:
 
     def __init__(self, name: str) -> None:
         self._commands: list[Command] = []
-        self._widget_map: dict[int, QCommandPalette] = {}
+        self._parent_to_palette_map: dict[int, QCommandPalette] = {}
+        self._palette_to_parent_map: WVDict = weakref.WeakValueDictionary()
         self._name = name
 
     @property
     def commands(self) -> list[Command]:
+        """List of all the commands."""
         return self._commands.copy()
 
     @overload
@@ -49,6 +61,7 @@ class CommandPalette:
         title: str | None,
         desc: str | None = None,
         tooltip: str | None = None,
+        when: Callable[[], bool] = _always_true,
     ) -> _F:
         ...
 
@@ -58,13 +71,12 @@ class CommandPalette:
         title: str | None,
         desc: str | None = None,
         tooltip: str | None = None,
+        when: Callable[[], bool] = _always_true,
     ) -> Callable[[_F], _F]:
         ...
 
     def register(self, *args, **kwargs):
-        """
-        Register a function to the command palette.
-        """
+        """Register a function to the command palette."""
         if len(args) > 0 and callable(args[0]):
             bound = register_with_func.bind(*args, **kwargs)
         else:
@@ -78,6 +90,7 @@ class CommandPalette:
         title = bound_args["title"]
         desc = bound_args["desc"]
         tooltip = bound_args["tooltip"]
+        when = bound_args["when"]
 
         if title is None:
             title = ""
@@ -92,28 +105,32 @@ class CommandPalette:
             storage = Storage.instance(self._name)
 
             @wraps(func)
-            def _func():
-                return storage.call(func)
+            def _func(qpallete):
+                parent = self._palette_to_parent_map[id(qpallete)]
+                return storage.call(func, parent)
 
-            cmd = Command(_func, title, desc, tooltip)
+            cmd = Command(_func, title, desc, tooltip, when)
             self._commands.append(cmd)
             return func
 
         return wrapper if func is None else wrapper(func)
 
     def add_group(self, title: str) -> CommandGroup:
+        """Add a group to the command palette."""
         return CommandGroup(title, parent=self)
 
     __default = object()
 
     def get_widget(self, parent: Any = __default) -> QCommandPalette:
+        """Get a command palette widget for the given parent widget."""
         from ._widget import QCommandPalette
 
         _id = id(parent)
-        if (widget := self._widget_map.get(_id)) is None:
+        if (widget := self._parent_to_palette_map.get(_id)) is None:
             widget = QCommandPalette()
             widget.extend_command(self._commands)
-            self._widget_map[_id] = widget
+            self._parent_to_palette_map[_id] = widget
+            self._palette_to_parent_map[id(widget)] = parent
         return widget
 
     def show_widget(self, parent: Any = __default) -> None:
@@ -138,6 +155,18 @@ class CommandPalette:
             _register_shortcut(keys, parent, lambda: self.show_widget(parent))
         return None
 
+    def update(self, parent: QtW.QWidget | None = None):
+        """Update command palette install to the given parent widget."""
+        if parent is None:
+            for p in self._palette_to_parent_map.values():
+                self.update(p)
+            return None
+        _id = id(parent)
+        widget = self._parent_to_palette_map[_id]
+        widget.clear_commands()
+        widget.extend_command(self._commands)
+        return None
+
 
 class CommandGroup:
     def __init__(self, title: str, parent: CommandPalette) -> None:
@@ -161,13 +190,20 @@ class CommandGroup:
 
     @overload
     def register(
-        self, func: _F, desc: str | None = None, tooltip: str | None = None
+        self,
+        func: _F,
+        desc: str | None = None,
+        tooltip: str | None = None,
+        when: Callable[[], bool] = _always_true,
     ) -> _F:
         ...
 
     @overload
     def register(
-        self, desc: str | None = None, tooltip: str | None = None
+        self,
+        desc: str | None = None,
+        tooltip: str | None = None,
+        when: Callable[[], bool] = _always_true,
     ) -> Callable[[_F], _F]:
         ...
 
@@ -240,6 +276,7 @@ def register(
     title: str | None,
     desc: str | None = None,
     tooltip: str | None = None,
+    when: Callable[[], bool] = _always_true,
 ) -> _F:
     ...
 
@@ -249,6 +286,7 @@ def register(
     title: str | None,
     desc: str | None = None,
     tooltip: str | None = None,
+    when: Callable[[], bool] = _always_true,
 ) -> Callable[[_F], _F]:
     ...
 
