@@ -9,10 +9,16 @@ from qtpy.QtCore import Qt, Signal
 from ._commands import Command
 
 logger = logging.getLogger(__name__)
+MATCH_COLOR = "blue"
+DISABLED_COLOR = "gray"
+
+
+def bold_colored(text: str, color: str) -> str:
+    return f"<b><font color={color!r}>{text}</font></b>"
 
 
 def colored(text: str, color: str) -> str:
-    return f"<b><font color={color!r}>{text}</font></b>"
+    return f"<font color={color!r}>{text}</font>"
 
 
 class QCommandMatchModel(QtCore.QAbstractListModel):
@@ -59,7 +65,7 @@ class QCommandLabel(QtW.QLabel):
         """The original command text."""
         return self._command_text
 
-    def set_text_colors(self, input_text: str, /, color: str = "blue"):
+    def set_text_colors(self, input_text: str, /, color: str = MATCH_COLOR):
         """Set label text color based on the input text."""
         if input_text == "":
             return None
@@ -72,7 +78,7 @@ class QCommandLabel(QtW.QLabel):
         for match_obj in pattern.finditer(text):
             output_texts.append(text[last_end : match_obj.start()])
             word = match_obj.group()
-            colored_word = colored(word, color)
+            colored_word = bold_colored(word, color)
             output_texts.append(colored_word)
             last_end = match_obj.end()
         output_texts.append(text[last_end:])
@@ -80,12 +86,19 @@ class QCommandLabel(QtW.QLabel):
         self.setText("".join(output_texts))
         return None
 
+    def set_disabled(self) -> None:
+        """Set the label to disabled."""
+        text = self.command_text()
+        self.setText(colored(text, DISABLED_COLOR))
+        return None
+
 
 class QCommandList(QtW.QListView):
     commandClicked = Signal(int)  # one of the items is clicked
 
     def __init__(self, parent: QtW.QWidget | None = None) -> None:
         super().__init__(parent)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.setModel(QCommandMatchModel(self))
         self.setSelectionMode(QtW.QAbstractItemView.SelectionMode.NoSelection)
         self._selected_index = 0
@@ -137,6 +150,10 @@ class QCommandList(QtW.QListView):
         self.all_commands.extend(commands)
         return None
 
+    def clear_commands(self) -> None:
+        """Clear all the command"""
+        return self.all_commands.clear()
+
     def command_at(self, index: int) -> Command:
         return self.indexWidget(self.model().index(index)).command()
 
@@ -155,10 +172,17 @@ class QCommandList(QtW.QListView):
             index = self._selected_index
         cmd = self.command_at(index)
         logger.debug(f"executing command: {cmd.fmt()}")
-        cmd()
+        cmd(self.parent())
         # move to the top
         self.all_commands.remove(cmd)
         self.all_commands.insert(0, cmd)
+        return None
+
+    def can_execute(self, index: int | None = None) -> bool:
+        if index is None:
+            index = self._selected_index
+        cmd = self.command_at(index)
+        return cmd.enabled()
 
     def update_for_text(self, input_text: str) -> None:
         """Update the list to match the input text."""
@@ -170,7 +194,10 @@ class QCommandList(QtW.QListView):
                 self.setRowHidden(row, False)
                 lw = self.indexWidget(self.model().index(row))
                 lw.set_command(cmd)
-                lw.set_text_colors(input_text, color=self._match_color)
+                if cmd.enabled():
+                    lw.set_text_colors(input_text, color=self._match_color)
+                else:
+                    lw.set_disabled()
                 row += 1
 
                 if row >= max_matches:
